@@ -32,6 +32,8 @@
 #define IWD_WSC_INTERFACE "net.connman.iwd.WiFiSimpleConfiguration"
 #define IWD_KNOWN_NETWORKS_INTERFACE "net.connman.iwd.KnownNetworks"
 #define IWD_SIGNAL_AGENT_INTERFACE "net.connman.iwd.SignalLevelAgent"
+#define IWD_AP_INTERFACE "net.connman.iwd.AccessPoint"
+#define IWD_ADHOC_INTERFACE "net.connman.iwd.AdHoc"
 
 #define IWD_AGENT_MANAGER_PATH "/"
 #define IWD_KNOWN_NETWORKS_PATH "/"
@@ -66,6 +68,14 @@ struct l_dbus_message *dbus_error_no_agent(struct l_dbus_message *msg);
 struct l_dbus_message *dbus_error_not_connected(struct l_dbus_message *msg);
 struct l_dbus_message *dbus_error_not_configured(struct l_dbus_message *msg);
 struct l_dbus_message *dbus_error_not_implemented(struct l_dbus_message *msg);
+struct l_dbus_message *dbus_error_service_set_overlap(
+						struct l_dbus_message *msg);
+struct l_dbus_message *dbus_error_already_provisioned(
+						struct l_dbus_message *msg);
+struct l_dbus_message *dbus_error_not_hidden(struct l_dbus_message *msg);
+
+struct l_dbus_message *dbus_error_from_errno(int err,
+						struct l_dbus_message *msg);
 
 bool dbus_init(bool enable_debug);
 bool dbus_exit(void);
