@@ -246,6 +246,10 @@ enum ie_rsn_akm_suite {
 	IE_RSN_AKM_SUITE_TDLS			= 0x0040,
 	IE_RSN_AKM_SUITE_SAE_SHA256		= 0x0080,
 	IE_RSN_AKM_SUITE_FT_OVER_SAE_SHA256	= 0x0100,
+	IE_RSN_AKM_SUITE_AP_PEER_KEY_SHA256	= 0x0200,
+	IE_RSN_AKM_SUITE_8021X_SUITE_B_SHA256	= 0x0400,
+	IE_RSN_AKM_SUITE_8021X_SUITE_B_SHA384	= 0x0800,
+	IE_RSN_AKM_SUITE_FT_OVER_8021X_SHA384	= 0x1000,
 };
 
 struct ie_tlv_iter {
