@@ -24,13 +24,6 @@
 
 struct device;
 
-enum security {
-	SECURITY_NONE,
-	SECURITY_WEP,
-	SECURITY_PSK,
-	SECURITY_8021X,
-};
-
 typedef void (*iwd_device_foreach_func)(struct device *, void *data);
 
 void __iwd_device_foreach(iwd_device_foreach_func func, void *user_data);
@@ -38,3 +31,12 @@ void __iwd_device_foreach(iwd_device_foreach_func func, void *user_data);
 const struct l_settings *iwd_get_config(void);
 
 void iwd_shutdown(void);
+
+void network_init();
+void network_exit();
+
+void sim_auth_init(void);
+void sim_auth_exit(void);
+
+bool wsc_init(void);
+bool wsc_exit();
